% this program finds the position and orientation of the motion platform
% when the lengths of the linear actuators are given.
% This is 'forward dynamics' for the Stewart platform, and involves the
% solution of 6 pretty-nonlinear equations in 6 variables at every point
% in time. We use MATLAB's fsolve (an iterative technique) to solve these ...
% We are doing the animation + nonlinear solution in 'real time' but not
% really optimizing for speed or anything. So occasionally the nonlinear
% solutions are slow, I imagine, near singular configurations. It would be
% useful to check for closeness to singularities in all these programs
% code by Manoj Srinivasan, Nov 2011.


% this would be useful if we had actuator lengths from sensors and wanted
% platform positions

clear all; close all; clc; % starting with a clean slate

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Defining the Stewart platform

% First, the 6 end-points of the linear actuator that connect to the base
radius_basepoints = 2.5; % constant radius for simplicity
% assuming a symmetric configuration for simplicity
alpha_B = pi/6; % base mounting angle
angles_basepoints = [0+alpha_B/2; 2*pi/3-alpha_B/2; 2*pi/3+alpha_B/2; ...
    4*pi/3-alpha_B/2; 4*pi/3+alpha_B/2; 0-alpha_B/2];
X_basepoints_0 = radius_basepoints*cos(angles_basepoints);
Y_basepoints_0 = radius_basepoints*sin(angles_basepoints);
Z_basepoints_0 = zeros(6,1);
figure(1); hold on; axis equal;
hbase = plot3([X_basepoints_0; X_basepoints_0(1)], ...
    [Y_basepoints_0; Y_basepoints_0(1)],[Z_basepoints_0; Z_basepoints_0(1)],'ro-');
Pos_basepoints_0 = [X_basepoints_0 Y_basepoints_0 Z_basepoints_0 ones(6,1)]'; % all points, in homogeneous coordinates
param.Pos_basepoints_0 = Pos_basepoints_0;

% Next the 6 end-points of the linear actuator that connect to the motion
% base (platform)
radius_platformpoints = 1;
alpha_P = pi/10;
angles_platformpoints = [pi/3-alpha_P/2; pi/3+alpha_P/2; ...
    pi-alpha_P/2; pi+alpha_P/2; ...
    5*pi/3-alpha_P/2; 5*pi/3+alpha_P/2];
X_platformpoints_3 = radius_platformpoints*cos(angles_platformpoints);
Y_platformpoints_3 = radius_platformpoints*sin(angles_platformpoints);
Z_platformpoints_3 = zeros(6,1);
Pos_platformpoints_3 = [X_platformpoints_3 Y_platformpoints_3 Z_platformpoints_3 ones(6,1)]'; % all points, in homogeneous coordinates
param.Pos_platformpoints_3 = Pos_platformpoints_3;

% initial vertical offset of the platform center from base center 
param.platform_height = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plotting the initial position of the Stewart platform

T_initial = homogeneous_transform(eye(3),[0; 0; param.platform_height]);

Pos_platformpoints_0 = T_initial*Pos_platformpoints_3;

X_platformpoints_0 = Pos_platformpoints_0(1,:)';
Y_platformpoints_0 = Pos_platformpoints_0(2,:)';
Z_platformpoints_0 = Pos_platformpoints_0(3,:)';

hplatform = patch([X_platformpoints_0; X_platformpoints_0(1)], ...
    [Y_platformpoints_0; Y_platformpoints_0(1)],[Z_platformpoints_0; Z_platformpoints_0(1)],'bo-');
view([1 1.3 0.3]);

% drawing the six actuators
ha1 = plot3([X_basepoints_0(1) X_platformpoints_0(1)], ...
    [Y_basepoints_0(1) Y_platformpoints_0(1)],[Z_basepoints_0(1) Z_platformpoints_0(1)],'k');
ha2 = plot3([X_basepoints_0(2) X_platformpoints_0(2)], ...
    [Y_basepoints_0(2) Y_platformpoints_0(2)],[Z_basepoints_0(2) Z_platformpoints_0(2)],'k');
ha3 = plot3([X_basepoints_0(3) X_platformpoints_0(3)], ...
    [Y_basepoints_0(3) Y_platformpoints_0(3)],[Z_basepoints_0(3) Z_platformpoints_0(3)],'k');
ha4 = plot3([X_basepoints_0(4) X_platformpoints_0(4)], ...
    [Y_basepoints_0(4) Y_platformpoints_0(4)],[Z_basepoints_0(4) Z_platformpoints_0(4)],'k');
ha5 = plot3([X_basepoints_0(5) X_platformpoints_0(5)], ...
    [Y_basepoints_0(5) Y_platformpoints_0(5)],[Z_basepoints_0(5) Z_platformpoints_0(5)],'k');
ha6 = plot3([X_basepoints_0(6) X_platformpoints_0(6)], ...
    [Y_basepoints_0(6) Y_platformpoints_0(6)],[Z_basepoints_0(6) Z_platformpoints_0(6)],'k');

% initial lengths
param.L1 = norm(Pos_platformpoints_0(1:3,1)-Pos_basepoints_0(1:3,1));
param.L2 = norm(Pos_platformpoints_0(1:3,2)-Pos_basepoints_0(1:3,2));
param.L3 = norm(Pos_platformpoints_0(1:3,3)-Pos_basepoints_0(1:3,3));
param.L4 = norm(Pos_platformpoints_0(1:3,4)-Pos_basepoints_0(1:3,4));
param.L5 = norm(Pos_platformpoints_0(1:3,5)-Pos_basepoints_0(1:3,5));
param.L6 = norm(Pos_platformpoints_0(1:3,6)-Pos_basepoints_0(1:3,6));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define the lengths of links as a functions of time
Ntimelist = 600;
tlist = linspace(0,3,Ntimelist)';

% default is the initial lengths list
length1list = param.L1*ones(Ntimelist,1); length2list = param.L2*ones(Ntimelist,1); length3list = param.L3*ones(Ntimelist,1);
length4list = param.L4*ones(Ntimelist,1); length5list = param.L5*ones(Ntimelist,1); length6list = param.L6*ones(Ntimelist,1);

% changing the lengths as some function of time % comment these out if you
% want the default lengths
length1list = param.L1+0.1*sin(4*tlist);
length2list = param.L2+0.2*sin(4*tlist);
length3list = param.L3+0.2*cos(4*tlist);
length4list = param.L4+0.1*sin(4*tlist);
length5list = param.L5+0.1*cos(4*tlist);
length6list = param.L6+0.1*sin(4*tlist);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Computing the position and orientation for the given length changes ...

platformconfig0 = [];

platformstate0 = [0; 0; 0; 0; 0; 0];

platformstate_list = zeros(6,Ntimelist);

for i = 1:Ntimelist
    
    i
    
    current_lengths = [length1list(i); length2list(i); length3list(i); length4list(i); ...
        length5list(i); length6list(i)];
    platformstate_now = find_platformconfig(current_lengths,platformstate0,param);
    platformstate0 = platformstate_now;
    
    platformstate_list(:,i) = platformstate_now; % recording the obtained states ...
    
    %%%% %% %% %% %% animation %%%%%%%%%%%%

    alphanow = platformstate_now(1);
    betanow = platformstate_now(2);
    gammanow = platformstate_now(3);
    xGnow = platformstate_now(4);
    yGnow = platformstate_now(5);
    zGnow = platformstate_now(6);
    
    Rot_total = rotxd(alphanow)*rotyd(betanow)*rotzd(gammanow);
    Disp_total = [xGnow; yGnow; zGnow+param.platform_height]; % adding the platform height ...
    
    Tran_total = homogeneous_transform(Rot_total,Disp_total);
    Pos_platformpoints_0 = Tran_total*Pos_platformpoints_3;

    X_platformpoints_0 = Pos_platformpoints_0(1,:)';
    Y_platformpoints_0 = Pos_platformpoints_0(2,:)';
    Z_platformpoints_0 = Pos_platformpoints_0(3,:)';

    set(hplatform,'xdata',[X_platformpoints_0; X_platformpoints_0(1)], ...
        'ydata',[Y_platformpoints_0; Y_platformpoints_0(1)], ...
        'zdata',[Z_platformpoints_0; Z_platformpoints_0(1)]);
%     view([1 1.3 0.3]);

    % drawing the six actuators
    set(ha1,'xdata',[X_basepoints_0(1) X_platformpoints_0(1)], ...
        'ydata',[Y_basepoints_0(1) Y_platformpoints_0(1)],...
        'zdata',[Z_basepoints_0(1) Z_platformpoints_0(1)]);
    set(ha2,'xdata',[X_basepoints_0(2) X_platformpoints_0(2)], ...
        'ydata',[Y_basepoints_0(2) Y_platformpoints_0(2)],...
        'zdata',[Z_basepoints_0(2) Z_platformpoints_0(2)]);
    set(ha3,'xdata',[X_basepoints_0(3) X_platformpoints_0(3)], ...
        'ydata',[Y_basepoints_0(3) Y_platformpoints_0(3)],...
        'zdata',[Z_basepoints_0(3) Z_platformpoints_0(3)]);
    set(ha4,'xdata',[X_basepoints_0(4) X_platformpoints_0(4)], ...
        'ydata',[Y_basepoints_0(4) Y_platformpoints_0(4)],...
        'zdata',[Z_basepoints_0(4) Z_platformpoints_0(4)]);
    set(ha5,'xdata',[X_basepoints_0(5) X_platformpoints_0(5)], ...
        'ydata',[Y_basepoints_0(5) Y_platformpoints_0(5)],...
        'zdata',[Z_basepoints_0(5) Z_platformpoints_0(5)]);
    set(ha6,'xdata',[X_basepoints_0(6) X_platformpoints_0(6)], ...
        'ydata',[Y_basepoints_0(6) Y_platformpoints_0(6)],...
        'zdata',[Z_basepoints_0(6) Z_platformpoints_0(6)]);
    
    % just to do a sanity check ...
    L1list(i) = norm(Pos_platformpoints_0(1:3,1)-Pos_basepoints_0(1:3,1));
    L2list(i) = norm(Pos_platformpoints_0(1:3,2)-Pos_basepoints_0(1:3,2));
    L3list(i) = norm(Pos_platformpoints_0(1:3,3)-Pos_basepoints_0(1:3,3));
    L4list(i) = norm(Pos_platformpoints_0(1:3,4)-Pos_basepoints_0(1:3,4));
    L5list(i) = norm(Pos_platformpoints_0(1:3,5)-Pos_basepoints_0(1:3,5));
    L6list(i) = norm(Pos_platformpoints_0(1:3,6)-Pos_basepoints_0(1:3,6));
    
    axis equal;
    axis([-2 2 -2 2 -2 2]);
    
    pause(0.005);
    %%%%%%%%%%%%%%%
    
%     pause
    
end

% just unpacking the stored results to individual vectors
xGlist = platformstate_list(3,:); yGlist = platformstate_list(4,:); zGlist = platformstate_list(5,:);
alphalist = platformstate_list(1,:); betalist = platformstate_list(2,:); gammalist = platformstate_list(3,:);

figure(2);

subplot(6,2,1); plot(tlist,L1list); ylabel('L1'); title('Forward kinematics INPUT: actuator lengths','interpreter','latex');
subplot(6,2,3); plot(tlist,L2list); ylabel('L2');
subplot(6,2,5); plot(tlist,L3list); ylabel('L3');
subplot(6,2,7); plot(tlist,L4list); ylabel('L4');
subplot(6,2,9); plot(tlist,L5list); ylabel('L5');
subplot(6,2,11); plot(tlist,L6list); ylabel('L6');

subplot(6,2,1); hold on; plot(tlist,length1list,'r'); 
subplot(6,2,3); hold on; plot(tlist,length2list,'r'); 
subplot(6,2,5); hold on; plot(tlist,length3list,'r'); 
subplot(6,2,7); hold on; plot(tlist,length4list,'r'); 
subplot(6,2,9); hold on; plot(tlist,length5list,'r'); 
subplot(6,2,11); hold on; plot(tlist,length6list,'r');

subplot(6,2,2); plot(tlist,xGlist); ylabel('xG'); title('Forward kinematics OUTPUT: platform motion','interpreter','latex');
subplot(6,2,4); plot(tlist,yGlist); ylabel('yG')
subplot(6,2,6); plot(tlist,zGlist); ylabel('zG')
subplot(6,2,8); plot(tlist,alphalist); ylabel('alpha'); 
subplot(6,2,10); plot(tlist,betalist);  ylabel('beta')
subplot(6,2,12); plot(tlist,gammalist); ylabel('gamma')

set(gcf, 'Units', 'normalized', 'Position', [0, 0, 0.7, 0.7]);