% this program computes the lengths of the 6 linear actuators, given the
% position and orientation of the base in some Euler angles, say
% code by Manoj Srinivasan, Nov 2011.

clear all; close all; clc; % starting with a clean slate

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Defining the MOTION of the PLATFORM.
% For this demo, I pick a 7-part motion: first pure x-motion, then pure-y
% motion, pure z-motion, pure X-rotation, pure Y-rotation, pure Z-rotation,
% and then some complex mixture of everything

Ntimelist = 7*100; % making it divisible by 7, so that we can do a 7-part motion
Ntime_eachpart = Ntimelist/7;

% Orientation of the motion base, in base coordinates, using "XYZ" Euler angles
% we may have to change the definition of the Euler angles to re-locate the singularity elsewhere
% Angles in degrees!

% zero defaults
xGlist_0 = zeros(Ntimelist,1); yGlist_0 = zeros(Ntimelist,1); zGlist_0 = zeros(Ntimelist,1); % zero defaults
alphalist_0 = zeros(Ntimelist,1); betalist_0 = zeros(Ntimelist,1); gammalist_0 = zeros(Ntimelist,1); % zero defaults

tlist = linspace(0,pi*7,Ntimelist);

% part-1: pure-x translation
xGlist_0(1:Ntime_eachpart) = 0.3*sin(tlist(1:Ntime_eachpart));

% part-2: pure-y translation
yGlist_0(Ntime_eachpart+1:2*Ntime_eachpart) = 0.4*sin(tlist(1:Ntime_eachpart)); %

% part-3: pure-z translation
zGlist_0(2*Ntime_eachpart+1:3*Ntime_eachpart) = 0.3*sin(tlist(1:Ntime_eachpart));

% part-4: pure X rotation
alphalist_0(3*Ntime_eachpart+1:4*Ntime_eachpart) = 5*sin(tlist(1:Ntime_eachpart));

% part-5: pure Y rotation
betalist_0(4*Ntime_eachpart+1:5*Ntime_eachpart) = 10*sin(tlist(1:Ntime_eachpart));

% part-6: pure Z rotation
gammalist_0(5*Ntime_eachpart+1:6*Ntime_eachpart) = 7*sin(tlist(1:Ntime_eachpart));

% part-7: some made-up complex 3D motion
xGlist_0(6*Ntime_eachpart+1:7*Ntime_eachpart) = 0.3*sin(3*tlist(1:Ntime_eachpart)); 
yGlist_0(6*Ntime_eachpart+1:7*Ntime_eachpart) = 0.2*sin(7*tlist(1:Ntime_eachpart)); 
zGlist_0(6*Ntime_eachpart+1:7*Ntime_eachpart) = 0.3*sin(2*tlist(1:Ntime_eachpart));
alphalist_0(6*Ntime_eachpart+1:7*Ntime_eachpart) = 5*cos(12*tlist(1:Ntime_eachpart)); 
betalist_0(6*Ntime_eachpart+1:7*Ntime_eachpart) = 9*sin(2*tlist(1:Ntime_eachpart)); 
gammalist_0(6*Ntime_eachpart+1:7*Ntime_eachpart) = 10*cos(3*tlist(1:Ntime_eachpart));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Defining the Stewart platform

% First, the 6 end-points of the linear actuator that connect to the base
radius_basepoints = 2.5; % constant radius for simplicity
alpha_B = pi/6; % base mounting angle
angles_basepoints = [0+alpha_B/2; 2*pi/3-alpha_B/2; 2*pi/3+alpha_B/2; ...
    4*pi/3-alpha_B/2; 4*pi/3+alpha_B/2; 0-alpha_B/2];
X_basepoints_0 = radius_basepoints*cos(angles_basepoints);
Y_basepoints_0 = radius_basepoints*sin(angles_basepoints);
Z_basepoints_0 = zeros(6,1);
figure(1); hold on; axis equal;
hbase = plot3([X_basepoints_0; X_basepoints_0(1)], ...
    [Y_basepoints_0; Y_basepoints_0(1)],[Z_basepoints_0; Z_basepoints_0(1)],'ro-');
Pos_basepoints_0 = [X_basepoints_0 Y_basepoints_0 Z_basepoints_0 ones(6,1)]'; % all points, in homogeneous coordinates

% Next the 6 end-points of the linear actuator that connect to the motion
% base (platform)
radius_platformpoints = 1;
alpha_P = pi/10;
angles_platformpoints = [pi/3-alpha_P/2; pi/3+alpha_P/2; ...
    pi-alpha_P/2; pi+alpha_P/2; ...
    5*pi/3-alpha_P/2; 5*pi/3+alpha_P/2];
X_platformpoints_3 = radius_platformpoints*cos(angles_platformpoints);
Y_platformpoints_3 = radius_platformpoints*sin(angles_platformpoints);
Z_platformpoints_3 = zeros(6,1);
Pos_platformpoints_3 = [X_platformpoints_3 Y_platformpoints_3 Z_platformpoints_3 ones(6,1)]'; % all points, in homogeneous coordinates

% initial vertical offset of the platform center from base center 
platform_height = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plotting the initial position of the Stewart platform

T_initial = homogeneous_transform(eye(3),[0; 0; platform_height]);

Pos_platformpoints_0 = T_initial*Pos_platformpoints_3;

X_platformpoints_0 = Pos_platformpoints_0(1,:)';
Y_platformpoints_0 = Pos_platformpoints_0(2,:)';
Z_platformpoints_0 = Pos_platformpoints_0(3,:)';

hplatform = patch([X_platformpoints_0; X_platformpoints_0(1)], ...
    [Y_platformpoints_0; Y_platformpoints_0(1)],[Z_platformpoints_0; Z_platformpoints_0(1)],'bo-');
view([1 1.3 0.4]);
xlabel('x'); ylabel('y'); zlabel('z');
title('A seven part motion: six elementary translations and rotations, then some comelex motion')

% drawing the six actuators
ha1 = plot3([X_basepoints_0(1) X_platformpoints_0(1)], ...
    [Y_basepoints_0(1) Y_platformpoints_0(1)],[Z_basepoints_0(1) Z_platformpoints_0(1)],'k');
ha2 = plot3([X_basepoints_0(2) X_platformpoints_0(2)], ...
    [Y_basepoints_0(2) Y_platformpoints_0(2)],[Z_basepoints_0(2) Z_platformpoints_0(2)],'k');
ha3 = plot3([X_basepoints_0(3) X_platformpoints_0(3)], ...
    [Y_basepoints_0(3) Y_platformpoints_0(3)],[Z_basepoints_0(3) Z_platformpoints_0(3)],'k');
ha4 = plot3([X_basepoints_0(4) X_platformpoints_0(4)], ...
    [Y_basepoints_0(4) Y_platformpoints_0(4)],[Z_basepoints_0(4) Z_platformpoints_0(4)],'k');
ha5 = plot3([X_basepoints_0(5) X_platformpoints_0(5)], ...
    [Y_basepoints_0(5) Y_platformpoints_0(5)],[Z_basepoints_0(5) Z_platformpoints_0(5)],'k');
ha6 = plot3([X_basepoints_0(6) X_platformpoints_0(6)], ...
    [Y_basepoints_0(6) Y_platformpoints_0(6)],[Z_basepoints_0(6) Z_platformpoints_0(6)],'k');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% initializing lengths to zero
L1list = zeros(Ntimelist,1); L2list = zeros(Ntimelist,1);
L3list = zeros(Ntimelist,1); L4list = zeros(Ntimelist,1);
L5list = zeros(Ntimelist,1); L6list = zeros(Ntimelist,1);

for i = 1:Ntimelist
    
    Rot_total = rotxd(alphalist_0(i))*rotyd(betalist_0(i))*rotzd(gammalist_0(i));
    Disp_total = [xGlist_0(i); yGlist_0(i); zGlist_0(i)+platform_height]; % adding the platform height ...
    
    Tran_total = homogeneous_transform(Rot_total,Disp_total);
    Pos_platformpoints_0 = Tran_total*Pos_platformpoints_3;

        
    X_platformpoints_0 = Pos_platformpoints_0(1,:)';
    Y_platformpoints_0 = Pos_platformpoints_0(2,:)';
    Z_platformpoints_0 = Pos_platformpoints_0(3,:)';

    set(hplatform,'xdata',[X_platformpoints_0; X_platformpoints_0(1)], ...
        'ydata',[Y_platformpoints_0; Y_platformpoints_0(1)],...
        'zdata',[Z_platformpoints_0; Z_platformpoints_0(1)]);
%     view([1 1.3 0.3]);

    % drawing the six actuators
    set(ha1,'xdata',[X_basepoints_0(1) X_platformpoints_0(1)], ...
        'ydata',[Y_basepoints_0(1) Y_platformpoints_0(1)],...
        'zdata',[Z_basepoints_0(1) Z_platformpoints_0(1)]);
    set(ha2,'xdata',[X_basepoints_0(2) X_platformpoints_0(2)], ...
        'ydata',[Y_basepoints_0(2) Y_platformpoints_0(2)],...
        'zdata',[Z_basepoints_0(2) Z_platformpoints_0(2)]);
    set(ha3,'xdata',[X_basepoints_0(3) X_platformpoints_0(3)], ...
        'ydata',[Y_basepoints_0(3) Y_platformpoints_0(3)],...
        'zdata',[Z_basepoints_0(3) Z_platformpoints_0(3)]);
    set(ha4,'xdata',[X_basepoints_0(4) X_platformpoints_0(4)], ...
        'ydata',[Y_basepoints_0(4) Y_platformpoints_0(4)],...
        'zdata',[Z_basepoints_0(4) Z_platformpoints_0(4)]);
    set(ha5,'xdata',[X_basepoints_0(5) X_platformpoints_0(5)], ...
        'ydata',[Y_basepoints_0(5) Y_platformpoints_0(5)],...
        'zdata',[Z_basepoints_0(5) Z_platformpoints_0(5)]);
    set(ha6,'xdata',[X_basepoints_0(6) X_platformpoints_0(6)], ...
        'ydata',[Y_basepoints_0(6) Y_platformpoints_0(6)],...
        'zdata',[Z_basepoints_0(6) Z_platformpoints_0(6)]);
    
    L1list(i) = norm(Pos_platformpoints_0(1:3,1)-Pos_basepoints_0(1:3,1));
    L2list(i) = norm(Pos_platformpoints_0(1:3,2)-Pos_basepoints_0(1:3,2));
    L3list(i) = norm(Pos_platformpoints_0(1:3,3)-Pos_basepoints_0(1:3,3));
    L4list(i) = norm(Pos_platformpoints_0(1:3,4)-Pos_basepoints_0(1:3,4));
    L5list(i) = norm(Pos_platformpoints_0(1:3,5)-Pos_basepoints_0(1:3,5));
    L6list(i) = norm(Pos_platformpoints_0(1:3,6)-Pos_basepoints_0(1:3,6));
    
    axis equal;
    axis([-2 2 -2 2 -2 2]);
    
    pause(0.005);
    
end


figure(2); 
subplot(6,2,1); plot(tlist,xGlist_0); ylabel('xG'); title('Inverse kinematics INPUT: platform motion','interpreter','latex');
subplot(6,2,3); plot(tlist,yGlist_0); ylabel('yG')
subplot(6,2,5); plot(tlist,zGlist_0); ylabel('zG')
subplot(6,2,7); plot(tlist,alphalist_0); ylabel('alpha'); 
subplot(6,2,9); plot(tlist,betalist_0);  ylabel('beta')
subplot(6,2,11); plot(tlist,gammalist_0); ylabel('gamma')
subplot(6,2,2); plot(tlist,L1list); ylabel('L1'); title('Inverse kinematics OUTPUT: actuator lengths','interpreter','latex');
subplot(6,2,4); plot(tlist,L2list); ylabel('L2');
subplot(6,2,6); plot(tlist,L3list); ylabel('L3');
subplot(6,2,8); plot(tlist,L4list); ylabel('L4');
subplot(6,2,10); plot(tlist,L5list); ylabel('L5');
subplot(6,2,12); plot(tlist,L6list); ylabel('L6');
xlabel('time');
set(gcf, 'Units', 'normalized', 'Position', [0, 0, 0.7, 0.7])