% this program computes the reachable workspace of the center of the
% Stewart platform, given bounds on actuator link lengths, without worrying
% about the platform orientation in which a particular point is reached.
% initial code by Manoj Srinivasan, Nov 2011.

clear all; close all; clc; % starting with a clean slate

% I need to do a little bit of work so that the Stewart platform may be
% defined based on the actuator link lengths: but you can scroll down if
% you need to change the mininimum and maximum of the linear actuator link
% lengths: param.L1min, param.L1max, etc. 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Defining the Stewart platform

% First, the 6 end-points of the linear actuator that connect to the base
radius_basepoints = 1.8; % constant radius for simplicity
% assuming a symmetric configuration for simplicity
alpha_B = pi/6; % base mounting angle
angles_basepoints = [0+alpha_B/2; 2*pi/3-alpha_B/2; 2*pi/3+alpha_B/2; ...
    4*pi/3-alpha_B/2; 4*pi/3+alpha_B/2; 0-alpha_B/2];X_basepoints_0 = radius_basepoints*cos(angles_basepoints);
Y_basepoints_0 = radius_basepoints*sin(angles_basepoints);
Z_basepoints_0 = zeros(6,1);
figure(1); hold on; axis equal;
hbase = plot3([X_basepoints_0; X_basepoints_0(1)], ...
    [Y_basepoints_0; Y_basepoints_0(1)],[Z_basepoints_0; Z_basepoints_0(1)],'ro-');
Pos_basepoints_0 = [X_basepoints_0 Y_basepoints_0 Z_basepoints_0 ones(6,1)]'; % all points, in homogeneous coordinates


% Next the 6 end-points of the linear actuator that connect to the motion
% base (platform)
radius_platformpoints = 1.5;
alpha_P = pi/10;
angles_platformpoints = [pi/3-alpha_P/2; pi/3+alpha_P/2; ...
    pi-alpha_P/2; pi+alpha_P/2; ...
    5*pi/3-alpha_P/2; 5*pi/3+alpha_P/2];
X_platformpoints_3 = radius_platformpoints*cos(angles_platformpoints);
Y_platformpoints_3 = radius_platformpoints*sin(angles_platformpoints);
Z_platformpoints_3 = zeros(6,1);
Pos_platformpoints_3 = [X_platformpoints_3 Y_platformpoints_3 Z_platformpoints_3 ones(6,1)]'; % all points, in homogeneous coordinates

% initial vertical offset of the platform center from base center 
platform_height = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plotting the initial position of the Stewart platform

T_initial = homogeneous_transform(eye(3),[0; 0; platform_height]);

Pos_platformpoints_0 = T_initial*Pos_platformpoints_3;

X_platformpoints_0 = Pos_platformpoints_0(1,:)';
Y_platformpoints_0 = Pos_platformpoints_0(2,:)';
Z_platformpoints_0 = Pos_platformpoints_0(3,:)';

hplatform = plot3([X_platformpoints_0; X_platformpoints_0(1)], ...
    [Y_platformpoints_0; Y_platformpoints_0(1)],[Z_platformpoints_0; Z_platformpoints_0(1)],'bo-');
view([1 1.3 0.6]);

% drawing the six actuators
ha1 = plot3([X_basepoints_0(1) X_platformpoints_0(1)], ...
    [Y_basepoints_0(1) Y_platformpoints_0(1)],[Z_basepoints_0(1) Z_platformpoints_0(1)],'k');
ha2 = plot3([X_basepoints_0(2) X_platformpoints_0(2)], ...
    [Y_basepoints_0(2) Y_platformpoints_0(2)],[Z_basepoints_0(2) Z_platformpoints_0(2)],'k');
ha3 = plot3([X_basepoints_0(3) X_platformpoints_0(3)], ...
    [Y_basepoints_0(3) Y_platformpoints_0(3)],[Z_basepoints_0(3) Z_platformpoints_0(3)],'k');
ha4 = plot3([X_basepoints_0(4) X_platformpoints_0(4)], ...
    [Y_basepoints_0(4) Y_platformpoints_0(4)],[Z_basepoints_0(4) Z_platformpoints_0(4)],'k');
ha5 = plot3([X_basepoints_0(5) X_platformpoints_0(5)], ...
    [Y_basepoints_0(5) Y_platformpoints_0(5)],[Z_basepoints_0(5) Z_platformpoints_0(5)],'k');
ha6 = plot3([X_basepoints_0(6) X_platformpoints_0(6)], ...
    [Y_basepoints_0(6) Y_platformpoints_0(6)],[Z_basepoints_0(6) Z_platformpoints_0(6)],'k');

% initial lengths
param.L1 = norm(Pos_platformpoints_0(1:3,1)-Pos_basepoints_0(1:3,1));
param.L2 = norm(Pos_platformpoints_0(1:3,2)-Pos_basepoints_0(1:3,2));
param.L3 = norm(Pos_platformpoints_0(1:3,3)-Pos_basepoints_0(1:3,3));
param.L4 = norm(Pos_platformpoints_0(1:3,4)-Pos_basepoints_0(1:3,4));
param.L5 = norm(Pos_platformpoints_0(1:3,5)-Pos_basepoints_0(1:3,5));
param.L6 = norm(Pos_platformpoints_0(1:3,6)-Pos_basepoints_0(1:3,6));

% minimum and maximum lengths of the actuator
% I'm just making them up for simplicity. But if you have actual values,
% you may enter them here ...
param.L1min = param.L1*0.8;
param.L2min = param.L2*0.8;
param.L3min = param.L3*0.8;
param.L4min = param.L4*0.8;
param.L5min = param.L5*0.8;
param.L6min = param.L6*0.8;

param.L1max = param.L1*1.2;
param.L2max = param.L2*1.2;
param.L3max = param.L3*1.2;
param.L4max = param.L4*1.2;
param.L5max = param.L5*1.2;
param.L6max = param.L6*1.2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Nlist = 30;
xGlist_0 = linspace(-0.6,0.6,Nlist);
yGlist_0 = linspace(-0.6,0.6,Nlist);
zGlist_0 = linspace(-0.6,0.6,Nlist);

pointlist = [];

for i = 1:Nlist
    for j = 1:Nlist
        for k = 1:Nlist
            
%             plot3(xGlist_0(i),yGlist_0(j),zGlist_0(k)+platform_height);
        
            Rot_total = eye(3);
            Disp_total = [xGlist_0(i); yGlist_0(j); zGlist_0(k)+platform_height]; % adding the platform height ...

            Tran_total = homogeneous_transform(Rot_total,Disp_total);
            Pos_platformpoints_0 = Tran_total*Pos_platformpoints_3;

            L1 = norm(Pos_platformpoints_0(1:3,1)-Pos_basepoints_0(1:3,1));
            L2 = norm(Pos_platformpoints_0(1:3,2)-Pos_basepoints_0(1:3,2));
            L3 = norm(Pos_platformpoints_0(1:3,3)-Pos_basepoints_0(1:3,3));
            L4 = norm(Pos_platformpoints_0(1:3,4)-Pos_basepoints_0(1:3,4));
            L5 = norm(Pos_platformpoints_0(1:3,5)-Pos_basepoints_0(1:3,5));
            L6 = norm(Pos_platformpoints_0(1:3,6)-Pos_basepoints_0(1:3,6));

            if (L1>param.L1min)&&(L1<param.L1max)&&(L2>param.L2min)&&(L2<param.L2max)&&...
                    (L3>param.L3min)&&(L3<param.L3max)&&(L4>param.L4min)&&(L4<param.L4max)...
                    &&(L5>param.L5min)&&(L5<param.L5max)&&(L6>param.L6min)&&(L6<param.L6max)


                pointlist = [pointlist; ...
                    xGlist_0(i) yGlist_0(j) zGlist_0(k)+platform_height];

        
            end
        end
    end
end

plot3(pointlist(:,1),pointlist(:,2),pointlist(:,3),'.');
set(gcf, 'Units', 'normalized', 'Position', [0, 0, 0.7, 0.7]);
htitle = title('Blue cloud: reachable workspace of platform center, with no rotation','interpreter','latex');
titlepos = get(htitle,'Position');
text(0,0,2,'Length limits for each actuator: 1.17 to 1.75 m');
text(0,0,2.5,'Use the rotate button above to visualize in 3D');

axis equal;
axis([-2 2 -2 2 -2 2]);
    